VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DocumentEditor"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Const ERROR_HIGHLIGHT_COLOR = wdViolet
Private Const COMMENT_PREFIX_BM = "zCOMMENT_TMP_"

Private document_ As Word.Document
Private highlightErrors_ As Boolean
Private commentUnresolved_ As Boolean

Private comments_ As Scripting.Dictionary

Public Function Init(target As Word.Document, iConfig As API_Config)
  Set document_ = target
  Set comments_ = New Scripting.Dictionary
  highlightErrors_ = iConfig.GetValue("HighlightErrors", False)
  commentUnresolved_ = iConfig.GetValue("CommentUnresolved", True)
End Function

Public Property Get Document() As Word.Document
  Set Document = document_
End Property

Public Function MarkError(target As Word.Range, sComment$)
'  
  If highlightErrors_ Then _
    Call FormatRange(target)
  If commentUnresolved_ Then _
    Call AddCommentToRange(target, sComment)
End Function

Public Function FormatRange(target As Word.Range)
'   
  Dim tmpRevision As Boolean: tmpRevision = document_.TrackRevisions
  document_.TrackRevisions = False
  target.HighlightColorIndex = ERROR_HIGHLIGHT_COLOR
  document_.TrackRevisions = tmpRevision
End Function

Public Function AddBookmark(target As Word.Range, sPrefix$) As String
  Dim nIndex&: nIndex = document_.Bookmarks.Count + 1
  Dim sBookmark$: sBookmark = sPrefix & nIndex
  Do While document_.Bookmarks.Exists(sBookmark)
    nIndex = nIndex - 1
    sBookmark = sPrefix & nIndex
  Loop
  
  AddBookmark = sBookmark
  If target.Bookmarks.Add(sBookmark, target) Is Nothing Then _
    Call Err.Raise(vbObjectError)
End Function

Public Function AddCommentToRange(target As Word.Range, sComment$)
'   (,   )
  Dim sBookmark$: sBookmark = AddBookmark(target, COMMENT_PREFIX_BM)
  If sBookmark = vbNullString Then _
    Exit Function
  Call comments_.Add(sBookmark, sComment)
End Function

Public Function AddGlobalComment(sComment$)
'       
  Dim iRange As Word.Range: Set iRange = document_.Paragraphs(1).Range
   If iRange.Comments.Count = 0 Then _
    Call iRange.Comments.Add(iRange)
  
  Dim theBM As Word.Comment: Set theBM = iRange.Comments(1)
  If theBM.Range.Text = vbNullString Then
    theBM.Range.Text = sComment
  Else
    theBM.Range.Text = theBM.Range.Text & vbNewLine & sComment
  End If
End Function

Public Function SetupFindLiteral(sSearch$) As Word.Range
  Dim rFind As Word.Range: Set rFind = document_.Range
  rFind.Find.ClearFormatting
  With rFind.Find
    .Text = sSearch
    .Format = True
    .Font.Hidden = False
    .MatchWildcards = False
    .Forward = True
  End With
  Set SetupFindLiteral = rFind
End Function

Public Function SetupFindWildcard(sSearch$) As Word.Range
  Dim rFind As Word.Range: Set rFind = document_.Range
  rFind.Find.ClearFormatting
  With rFind.Find
    .Text = sSearch
    .Format = True
    .Font.Hidden = False
    .MatchWildcards = True
    .Forward = True
  End With
  Set SetupFindWildcard = rFind
End Function

Public Function ReplaceText(sFind$, sReplacement$, bApplyFix As Boolean, Optional sComment$ = vbNullString) As Long
  ReplaceText = FindAndReplace(sFind, sReplacement, bApplyFix, sComment, _
                              bCase:=False, bWildcard:=False, bWhole:=True)
End Function

Public Function ReplaceWC(sFind$, sReplacement$, bApplyFix As Boolean, Optional sComment$ = vbNullString) As Long
  ReplaceWC = FindAndReplace(sFind, sReplacement, bApplyFix, sComment, _
                            bCase:=False, bWildcard:=True, bWhole:=False)
End Function

Public Function FindAndReplace(sFind$, sReplacement$, _
                bApplyFix As Boolean, Optional sComment$ = vbNullString, _
                Optional bCase As Boolean = False, _
                Optional bWildcard As Boolean = False, _
                Optional bWhole As Boolean = True) As Long
'        
  Dim result&:  result = 0
  Dim rFind As Word.Range: Set rFind = document_.Range
  With rFind.Find
    .Text = sFind
    .Replacement.Text = IIf(bApplyFix, sReplacement, "")
    .Format = True
    .Font.Hidden = False
    .MatchCase = bCase
    .MatchWholeWord = bWhole
    .MatchWildcards = bWildcard
    .Forward = True
  End With
  
  Do While rFind.Find.Execute(Replace:=1 * bApplyFix)
    result = result + 1
    If Not bApplyFix Then _
      Call MarkError(rFind, sComment)
    Call rFind.Collapse(wdCollapseEnd)
  Loop
  
  FindAndReplace = result
End Function

Public Function ReplaceCascade(sFind$, _
                findText2$, repText2$, bApplyFix As Boolean, _
                Optional sComment$ = vbNullString, _
                Optional bCase As Boolean = False, _
                Optional bWildcard As Boolean = False, _
                Optional bWhole As Boolean = False) As Long
'        bWildcard
'   ,          "\1"
  Dim result&: result = 0
  If repText2 = vbNullString And bApplyFix Then
    ReplaceCascade = result
    Debug.Print " :   "
    Exit Function
  End If
  
  Dim rFind As Word.Range: Set rFind = document_.Range
  With rFind.Find
    .Text = sFind
    .MatchWildcards = True
    .Forward = True
    .Format = True
    .Font.Hidden = False
    
    Do While .Execute
      .Text = findText2
      .Replacement.Text = repText2
      .MatchCase = bCase
      .MatchWholeWord = bWhole
      .MatchWildcards = bWildcard
      
      If .Execute(Replace:=1 * bApplyFix) Then
        result = result + 1
        If Not bApplyFix Then _
          Call MarkError(rFind, sComment)
      End If
      
      .Text = sFind
      .Replacement.Text = vbNullString
      .MatchCase = False
      .MatchWholeWord = False
      .MatchWildcards = True
      
      Call rFind.Collapse(wdCollapseEnd)
    Loop
  End With
  
  ReplaceCascade = result
End Function

Public Function FinalizeComments()
'    (   )
  On Error Resume Next
  If document_.Bookmarks.Count = 0 Then _
    Exit Function
  
  If CSE_ProgressBar.Visible Then _
    Call CSE_ProgressBar.Init("", sHeader:="  ...", minVal:=0, curVal:=1, maxVal:=comments_.Count)
  
  Dim target As Word.Range
  Dim vBookmark As Variant
  For Each vBookmark In comments_.Keys()
    If Not document_.Bookmarks.Exists(CStr(vBookmark)) Then _
      GoTo NEXT_ITEM
    Set target = document_.Bookmarks(vBookmark).Range
    Call document_.Bookmarks(vBookmark).Delete
    Call target.Comments.Add(target, comments_(vBookmark))
NEXT_ITEM:
    If CSE_ProgressBar.Visible Then _
      Call CSE_ProgressBar.IncrementA
  Next
  
  Set comments_ = New Scripting.Dictionary
End Function

